@@ -38,12 +38,9 @@ CONNECTOR_TYPE = "google_drive"
 
 # Maps Google-native Drive MIME types → export MIME types
 GOOGLE_EXPORT_MIME_MAP = {
-    "application/vnd.google-apps.document": \
-        "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
-    "application/vnd.google-apps.spreadsheet": \
-        "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
-    "application/vnd.google-apps.presentation": \
-        "application/vnd.openxmlformats-officedocument.presentationml.presentation",
+    "application/vnd.google-apps.document": "application/vnd.openxmlformats-officedocument.wordprocessingml.document",  # noqa: E501
+    "application/vnd.google-apps.spreadsheet": "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",  # noqa: E501
+    "application/vnd.google-apps.presentation": "application/vnd.openxmlformats-officedocument.presentationml.presentation",  # noqa: E501
 }
 
 # Maps export MIME types → file extensions
@@ -55,11 +52,15 @@ EXPORT_EXTENSION_MAP = {
     "text/html": ".html",
 }
 
+# LRO Export Size Threshold is 10MB in real but the exported file might be slightly larger
+# than the original Google Workspace file - thus the threshold is set to 9MB
+LRO_EXPORT_SIZE_THRESHOLD = 9 * 1024 * 1024  # 9MB
+
 
 class GoogleDriveAccessConfig(AccessConfig):
-    service_account_key: Optional[
-        Annotated[dict, BeforeValidator(conform_string_to_dict)]
-    ] = Field(default=None, description="Credentials values to use for authentication")
+    service_account_key: Optional[Annotated[dict, BeforeValidator(conform_string_to_dict)]] = Field(
+        default=None, description="Credentials values to use for authentication"
+    )
     service_account_key_path: Optional[Path] = Field(
         default=None,
         description="File path to credentials values to use for authentication",
@@ -145,8 +146,7 @@ class GoogleDriveIndexer(Indexer):
             "originalFilename",
             "capabilities",
             "permissionIds",
-            "webViewLink",
-            "webContentLink",
+            "size",
         ]
     )
 
@@ -178,13 +178,11 @@ class GoogleDriveIndexer(Indexer):
                     Please enable it in the Google Cloud Console."
                 )
             else:
-                raise SourceConnectionError(
-                    "Google drive API unreachable for an unknown reason!"
-                )
+                raise SourceConnectionError("Google drive API unreachable for an unknown reason!")
 
     @staticmethod
     def count_files_recursively(
-        files_client, folder_id: str, extensions: list[str] = None
+        files_client: "GoogleAPIResource", folder_id: str, extensions: list[str] = None
     ) -> int:
         """
         Count non-folder files recursively under the given folder.
@@ -266,9 +264,7 @@ class GoogleDriveIndexer(Indexer):
                         #     that the service account has proper permissions."
                         # )
                     else:
-                        logger.info(
-                            f"Found {file_count} files recursively in the folder."
-                        )
+                        logger.info(f"Found {file_count} files recursively in the folder.")
                 else:
                     # Non-recursive: check for at least one immediate non-folder child.
                     response = client.list(
@@ -319,9 +315,7 @@ class GoogleDriveIndexer(Indexer):
         date_modified_str = root_info.pop("modifiedTime", None)
         parent_path = root_info.pop("parent_path", None)
         parent_root_path = root_info.pop("parent_root_path", None)
-        date_modified_dt = (
-            parser.parse(date_modified_str) if date_modified_str else None
-        )
+        date_modified_dt = parser.parse(date_modified_str) if date_modified_str else None
         if (
             parent_path
             and isinstance(parent_path, str)
@@ -350,6 +344,7 @@ class GoogleDriveIndexer(Indexer):
                 },
             ),
             additional_metadata=root_info,
+            display_name=source_identifiers.fullpath,
         )
 
     def get_paginated_results(
@@ -421,9 +416,7 @@ class GoogleDriveIndexer(Indexer):
     ) -> list[FileData]:
         root_info = self.get_root_info(files_client=files_client, object_id=object_id)
         if not self.is_dir(root_info):
-            root_info["permissions"] = self.extract_permissions(
-                root_info.get("permissions")
-            )
+            root_info["permissions"] = self.extract_permissions(root_info.get("permissions"))
             data = [self.map_file_data(root_info)]
         else:
             file_contents = self.get_paginated_results(
@@ -490,22 +483,26 @@ class GoogleDriveIndexer(Indexer):
 
 
 class GoogleDriveDownloaderConfig(DownloaderConfig):
-    pass
+    lro_max_tries: int = 10
+    lro_max_time: int = 10 * 60  # 10 minutes
+
+
+def _get_extension(file_data: FileData) -> str:
+    """
+    Returns the extension for a given source MIME type.
+    """
+    source_mime_type = file_data.additional_metadata.get("export_mime_type", "")
+    export_mime_type = GOOGLE_EXPORT_MIME_MAP.get(source_mime_type, "")
+    if export_mime_type:
+        return EXPORT_EXTENSION_MAP.get(export_mime_type, "")
+    return ""
 
 
 @dataclass
 class GoogleDriveDownloader(Downloader):
     """
-    Downloads files from Google Drive using authenticated direct HTTP requests
-    via `exportLinks` (for Google-native files) and `webContentLink` (for binary files).
-
-    These links emulate the behavior of Google Drive's "File > Download as..." options
-    in the UI and bypass the size limitations of `files.export()`.
-
-    Behavior:
-    - Google-native formats are downloaded using `exportLinks` in appropriate MIME formats.
-    - Binary files (non-Google-native) are downloaded using `webContentLink`.
-    - All downloads are performed via `requests.get()` using a valid bearer token.
+    Downloads files from Google Drive using googleapis client. For native files, it uses the export
+    functionality for files <10MB and LRO (Long Running Operation) for files >10MB.
     """
 
     connection_config: GoogleDriveConnectionConfig
@@ -514,81 +511,233 @@ class GoogleDriveDownloader(Downloader):
     )
     connector_type: str = CONNECTOR_TYPE
 
-    def _get_download_url_and_ext(
-        self, file_id: str, mime_type: str
-    ) -> tuple[str, str]:
-        """
-        Resolves the appropriate download URL and expected file extension for a Google Drive file.
+    @requires_dependencies(["googleapiclient"], extras="google-drive")
+    def _direct_download_file(self, file_id, download_path: Path):
+        """Downloads a file from Google Drive using the Drive API's media download functionality.
+        The method uses Google Drive API's media download functionality to stream the file
+        content directly to disk.
 
-        - Google-native files use export MIME types from exportLinks (e.g., .docx, .xlsx).
-        - Binary files use webContentLink (e.g., uploaded PDFs or ZIPs).
-
-        Returns:
-            Tuple[str, str]: (download URL, file extension or "")
+        Args:
+            file_id (str): The ID of the file to download from Google Drive.
+            download_path (Path): The local path where the file should be saved.
 
         Raises:
-            SourceConnectionError: If no valid export or download link is available.
+            SourceConnectionError: If the download operation fails.
         """
-        with self.connection_config.get_client() as client:
-            metadata = client.get(
-                fileId=file_id, fields="exportLinks,webContentLink"
-            ).execute()
+        from googleapiclient.errors import HttpError
+        from googleapiclient.http import MediaIoBaseDownload
 
-        export_links = metadata.get("exportLinks", {})
-        web_link = metadata.get("webContentLink")
+        try:
+            with self.connection_config.get_client() as client:
+                # pylint: disable=maybe-no-member
+                request = client.get_media(fileId=file_id)
 
-        if export_mime := GOOGLE_EXPORT_MIME_MAP.get(mime_type):
-            url = export_links.get(export_mime)
-            if not url:
-                raise SourceConnectionError(
-                    f"No export link found for {file_id} as {export_mime}"
-                )
-            ext = EXPORT_EXTENSION_MAP.get(export_mime, "")
-            return url, ext
+                with open(download_path, "wb") as file:
+                    downloader = MediaIoBaseDownload(file, request)
+                    done = False
+                    while done is False:
+                        status, done = downloader.next_chunk()
+                        logger.debug(f"Download progress:{int(status.progress() * 100)}.")
 
-        if not web_link:
-            raise SourceConnectionError(
-                f"No webContentLink available for file {file_id}"
+        except (HttpError, ValueError) as error:
+            logger.exception(f"Error downloading file {file_id} to {download_path}: {error}")
+            raise SourceConnectionError("Failed to download file") from error
+
+    @requires_dependencies(["googleapiclient"], extras="google-drive")
+    def _export_gdrive_file_with_lro(self, file_id: str, download_path: Path, mime_type: str):
+        """Exports a Google Drive file using Long-Running Operation (LRO) for large files
+        (>10MB of the exported file size).
+
+        This method is used when the standard export method fails due to file size limitations.
+        It uses the Drive API's LRO functionality to handle large file exports.
+
+        Args:
+            file_id (str): The ID of the Google Drive file to export.
+            download_path (Path): The local path where the exported file should be saved.
+            mime_type (str): The target MIME type for the exported file.
+        Raises:
+            SourceConnectionError: If the export operation fails.
+        """
+
+        import tenacity
+        from googleapiclient.errors import HttpError
+
+        max_time = self.download_config.lro_max_time
+        max_tries = self.download_config.lro_max_tries
+
+        class OperationNotFinished(Exception):
+            """
+            Exception raised when the operation is not finished.
+            """
+
+            pass
+
+        def is_fatal_code(e: Exception) -> bool:
+            """
+            Returns True if the error is fatal and should not be retried.
+            403 and 429 can mean "Too many requests" or "User rate limit exceeded"
+            which should be retried.
+            """
+            return (
+                isinstance(e, HttpError)
+                and 400 <= e.resp.status < 500
+                and e.resp.status not in [403, 429]
             )
-        return web_link, ""
 
-    @requires_dependencies(["httpx", "google.auth"], extras="google-drive")
-    def _download_url(self, file_data: FileData, url: str, ext: str = "") -> Path:
+        @tenacity.retry(
+            wait=tenacity.wait_exponential(),
+            retry=tenacity.retry_if_exception(
+                lambda e: (
+                    isinstance(e, (HttpError, OperationNotFinished)) and not is_fatal_code(e)
+                )
+            ),
+            stop=(tenacity.stop_after_attempt(max_tries) | tenacity.stop_after_delay(max_time)),
+        )
+        def _poll_operation(operation: dict, operations_client: "GoogleAPIResource") -> dict:
+            """
+            Helper function to poll the operation until it's complete.
+            Uses backoff exponential retry logic.
+
+            Each `operations.get` call uses the Google API requests limit. Details:
+            https://developers.google.com/workspace/drive/api/guides/limits
+
+            The limits as of May 2025 are:
+            - 12.000 calls per 60 seconds
+
+            In case of request limitting, the API will return 403 `User rate limit exceeded` error
+            or 429 `Too many requests` error.
+            """
+            if operation.get("done", False):
+                return operation
+            if "error" in operation:
+                raise SourceConnectionError(
+                    f"Export operation failed: {operation['error']['message']}"
+                )
+            # Refresh the operation status:
+            # FYI: In some cases the `operations.get` call errors with 403 "User does not have
+            # permission" error even if the same user create the operation with `download` method.
+            updated_operation = operations_client.get(name=operation["name"]).execute()
+            if not updated_operation.get("done", False):
+                raise OperationNotFinished()
+            return updated_operation
+
+        try:
+            with self._get_files_and_operations_client() as (files_client, operations_client):
+                # Start the LRO
+                operation = files_client.download(fileId=file_id, mimeType=mime_type).execute()
+
+                # In case the operation is not finished, poll it until it's complete
+                updated_operation = _poll_operation(operation, operations_client)
+
+                # Get the download URI from the completed operation
+                download_uri = updated_operation["response"]["downloadUri"]
+
+            # Download the file using the URI
+            self._raw_download_google_drive_file(download_uri, download_path)
+
+        except HttpError as error:
+            raise SourceConnectionError(
+                f"Failed to export file using Google Drive LRO: {error}"
+            ) from error
+
+    @requires_dependencies(["googleapiclient"], extras="google-drive")
+    def _export_gdrive_native_file(
+        self, file_id: str, download_path: Path, mime_type: str, file_size: int
+    ):
+        """Exports a Google Drive native file (Docs, Sheets, Slides) to a specified format.
+
+        This method uses the Google Drive API's export functionality to convert Google Workspace
+        files to other formats (e.g., Google Docs to PDF, Google Sheets to Excel).
+        For files larger than 10MB, it falls back to using Long-Running Operation (LRO).
+
+        Args:
+            file_id (str): The ID of the Google Drive file to export.
+            download_path (Path): The local path where the exported file should be saved.
+            mime_type (str): The target MIME type for the exported file (e.g., 'application/pdf').
+            file_size (int): The size of the file to export - used to determine if the
+                file is large enough to use LRO instead of direct export endpoint.
+        Returns:
+            bytes: The exported file content.
+
+        Raises:
+            HttpError: If the export operation fails.
+        """
+        from googleapiclient.errors import HttpError
+        from googleapiclient.http import MediaIoBaseDownload
+
+        if file_size > LRO_EXPORT_SIZE_THRESHOLD:
+            self._export_gdrive_file_with_lro(file_id, download_path, mime_type)
+            return
+
+        with self.connection_config.get_client() as client:
+            try:
+                # pylint: disable=maybe-no-member
+                request = client.export_media(fileId=file_id, mimeType=mime_type)
+                with open(download_path, "wb") as file:
+                    downloader = MediaIoBaseDownload(file, request)
+                    done = False
+                    while done is False:
+                        status, done = downloader.next_chunk()
+                        logger.debug(f"Download progress: {int(status.progress() * 100)}.")
+            except HttpError as error:
+                if error.resp.status == 403 and "too large" in error.reason.lower():
+                    # Even though we have the LRO threashold, for some smaller files the
+                    # export size might exceed 10MB and we get a 403 error.
+                    # In that case, we use LRO as a fallback.
+                    self._export_gdrive_file_with_lro(file_id, download_path, mime_type)
+                else:
+                    raise SourceConnectionError(f"Failed to export file: {error}") from error
+
+    @requires_dependencies(["googleapiclient"], extras="google-drive")
+    @contextmanager
+    def _get_files_and_operations_client(
+        self,
+    ) -> Generator[tuple["GoogleAPIResource", "GoogleAPIResource"], None, None]:
+        """
+        Returns a context manager for the files and operations clients for the Google Drive API.
+
+        Yields:
+            Tuple[GoogleAPIResource, GoogleAPIResource]: A tuple of the files
+                and operations clients.
+        """
+        from googleapiclient.discovery import build
+
+        creds = self._get_credentials()
+        service = build("drive", "v3", credentials=creds)
+        with (
+            service.operations() as operations_client,
+            service.files() as files_client,
+        ):
+            yield files_client, operations_client
+
+    @requires_dependencies(["httpx"])
+    def _raw_download_google_drive_file(self, url: str, download_path: Path) -> Path:
         """
         Streams file content directly to disk using authenticated HTTP request.
+        Must use httpx to stream the file to disk as currently there's no google SDK
+        functionality to download a file like for get media or export operations.
 
         Writes the file to the correct path in the download directory while downloading.
         Avoids buffering large files in memory.
 
-        Returns:
-            Path to the downloaded file.
+        Args:
+            url (str): The URL of the file to download.
+            download_path (Path): The path to save the downloaded file.
 
-        Raises:
-            SourceConnectionError: If the HTTP request fails.
+        Returns:
+            Path: The path to the downloaded file.
         """
         import httpx
         from google.auth.transport.requests import Request
-        from google.oauth2 import service_account
 
-        access_config = self.connection_config.access_config.get_secret_value()
-        key_data = access_config.get_service_account_key()
-        creds = service_account.Credentials.from_service_account_info(
-            key_data,
-            scopes=["https://www.googleapis.com/auth/drive.readonly"],
-        )
+        creds = self._get_credentials()
+
         creds.refresh(Request())
 
         headers = {
             "Authorization": f"Bearer {creds.token}",
         }
 
-        download_path = self.get_download_path(file_data)
-        if ext:
-            download_path = download_path.with_suffix(ext)
-
-        download_path.parent.mkdir(parents=True, exist_ok=True)
-        logger.debug(f"Streaming file to {download_path}")
-
         with (
             httpx.Client(timeout=None, follow_redirects=True) as client,
             client.stream("GET", url, headers=headers) as response,
@@ -600,31 +749,94 @@ class GoogleDriveDownloader(Downloader):
             with open(download_path, "wb") as f:
                 for chunk in response.iter_bytes():
                     f.write(chunk)
+        return download_path
+
+    @requires_dependencies(["google"], extras="google-drive")
+    def _get_credentials(self):
+        """
+        Retrieves the credentials for Google Drive API access.
+
+        Returns:
+            Credentials: The credentials for Google Drive API access.
+        """
+        from google.oauth2 import service_account
+
+        access_config = self.connection_config.access_config.get_secret_value()
+        key_data = access_config.get_service_account_key()
+        creds = service_account.Credentials.from_service_account_info(
+            key_data,
+            scopes=["https://www.googleapis.com/auth/drive.readonly"],
+        )
+        return creds
+
+    def _download_file(self, file_data: FileData) -> Path:
+        """Downloads a file from Google Drive using either direct download or export based
+        on the source file's MIME type.
+
+        This method determines the appropriate download method based on the file's MIME type:
+        - For Google Workspace files (Docs, Sheets, Slides), uses export functionality
+        - For other files, uses direct download
+
+        Args:
+            file_data (FileData): The metadata of the file being downloaded.
+
+        Returns:
+            Path: The path to the downloaded file.
+
+        Raises:
+            SourceConnectionError: If the download fails.
+        """
+        mime_type = file_data.additional_metadata.get("mimeType", "")
+        file_size = int(file_data.additional_metadata.get("size", 0))
+        file_id = file_data.identifier
+
+        download_path = self.get_download_path(file_data)
+        if not download_path:
+            raise SourceConnectionError(f"Failed to get download path for file {file_id}")
+
+        if mime_type in GOOGLE_EXPORT_MIME_MAP:
+            # For Google Workspace files, use export functionality
+            ext = _get_extension(file_data)
+            download_path = download_path.with_suffix(ext)
+            download_path.parent.mkdir(parents=True, exist_ok=True)
+            export_mime = GOOGLE_EXPORT_MIME_MAP[mime_type]
+            self._export_gdrive_native_file(
+                file_id=file_id,
+                download_path=download_path,
+                mime_type=export_mime,
+                file_size=file_size,
+            )
+            file_data.additional_metadata.update(
+                {
+                    "export_mime_type": export_mime,
+                    "export_extension": ext,
+                    "download_method": "google_workspace_export",
+                }
+            )
+        else:
+            # For other files, use direct download
+            download_path.parent.mkdir(parents=True, exist_ok=True)
+            self._direct_download_file(file_id=file_id, download_path=download_path)
+            file_data.additional_metadata.update(
+                {
+                    "download_method": "direct_download",
+                }
+            )
 
         return download_path
 
     def run(self, file_data: FileData, **kwargs: Any) -> DownloadResponse:
         mime_type = file_data.additional_metadata.get("mimeType", "")
-        record_id = file_data.identifier
 
         logger.debug(
             f"Downloading file {file_data.source_identifiers.fullpath} of type {mime_type}"
         )
 
-        download_url, ext = self._get_download_url_and_ext(record_id, mime_type)
-        download_path = self._download_url(file_data, download_url, ext)
+        download_path = self._download_file(file_data)
 
-        file_data.additional_metadata.update(
-            {
-                "download_method": "export_link" if ext else "web_content_link",
-                "download_url_used": download_url,
-            }
-        )
         file_data.local_download_path = str(download_path.resolve())
 
-        return self.generate_download_response(
-            file_data=file_data, download_path=download_path
-        )
+        return self.generate_download_response(file_data=file_data, download_path=download_path)
 
 
 google_drive_source_entry = SourceRegistryEntry(
